import { Injectable, Logger } from '@nestjs/common';
import { EventEmitter2 } from '@nestjs/event-emitter';
import { SqlRepositoryBase } from '@src/libs/db/sql-repository.base';
import { InjectPool } from 'nestjs-slonik';
import { DatabasePool } from 'slonik';
import { z } from 'zod';
import { UserEntity } from '../domain/user.entity';
import { UserMapper } from '../user.mapper';
import { UserRepositoryPort } from './user.repository.port';

export const userSchema = z.object({
  id: z.string().uuid(),
  email: z.string().email(),
  phone_number: z.string().min(10).max(15),
  fullname: z.string(),
  avatar:  z.string(),
  latitude:  z.string(),
  longitude:  z.string(),
  auth_token:  z.string(),
  verify_token:  z.string(),
  fcm_token:  z.string(),
  is_verified:  z.string(),
  is_merchant:  z.string(),
  created_at: z.preprocess((val: any) => new Date(val), z.date()),
  updated_at: z.preprocess((val: any) => new Date(val), z.date()),
});


export type UserModel = z.TypeOf<typeof userSchema>;

/**
 *  Repository is used for retrieving/saving domain entities
 * */
@Injectable()
export class UserRepository 
  extends SqlRepositoryBase<UserEntity, UserModel> 
  implements UserRepositoryPort 
{
  findOneByEmail(email: string): Promise<UserEntity> {
    throw new Error('Method not implemented.');
  }
  protected tableName: 'users';
  protected schema = userSchema;

  constructor(
    @InjectPool()
    pool: DatabasePool,
    mapper: UserMapper,
    eventEmitter: EventEmitter2
  ) {
    super(pool, mapper, eventEmitter, new Logger(UserRepository.name));
  }

  async updateAddress(user: UserEntity): Promise<void> {
    const address = user.getPropsCopy().address;
  }
  
}